@@ -17,15 +17,16 @@ __date__ = "Mar 2 2013"
 
 import json
 import itertools
+import logging
 import os
 from collections import OrderedDict, Iterable
-
 from pymongo import MongoClient
-
 from pymatgen import Structure, Composition
 from pymatgen.entries.computed_entries import ComputedEntry,\
     ComputedStructureEntry
 
+_log = logging.getLogger('mg.' + __name__)
+
 
 class QueryEngine(object):
     """
@@ -47,9 +48,18 @@ class QueryEngine(object):
     accessed simply as doc["a.b"].
     """
 
+    # avoid hard-coding these in other places
+    ALIASES_CONFIG_KEY = 'aliases_config'
+    COLLECTION_KEY = 'collection'
+    HOST_KEY = 'host'
+    PORT_KEY = 'port'
+    DB_KEY = 'database'
+    USER_KEY = 'user'
+    PASSWORD_KEY = 'password'
+
     def __init__(self, host="127.0.0.1", port=27017, database="vasp",
                  user=None, password=None, collection="tasks",
-                 aliases_config=None, default_properties=None):
+                 aliases_config=None, default_properties=None, **ignore):
         """
         Args:
             host:
@@ -292,7 +302,12 @@ class QueryEngine(object):
                 parsed_crit[self.aliases.get(key, key)] = crit
         return parsed_crit
 
-    def query(self, properties=None, criteria=None, index=0, limit=None):
+    def ensure_index(self, key, unique=False):
+        """Wrapper for pymongo.Collection.ensure_index
+        """
+        return self.collection.ensure_index(key, unique=unique)
+
+    def query(self, properties=None, criteria=None, index=0, limit=None, distinct_key=None):
         """
         Convenience method for database access.  All properties and criteria
         can be specified using simplified names defined in Aliases.  You can
@@ -314,36 +329,31 @@ class QueryEngine(object):
         returned as r['analysis'] and then the subkeys can be accessed in the
         usual form, i.e., r['analysis']['e_above_hull']
 
-        Args:
-            properties:
-                Properties to query for. Defaults to None which means all
-                supported properties.
-            criteria:
-                Criteria to query for as a dict.
-            index:
-                Similar definition to pymongo.collection.find method.
-            limit:
-                Similar definition to pymongo.collection.find method.
-
-        Returns:
-            An QueryResults Iterable, which is somewhat like pymongo's
+        :param properties: Properties to query for. Defaults to None which means all supported properties.
+        :param criteria: Criteria to query for as a dict.
+        :parm index: Similar definition to pymongo.collection.find method.
+        :param limit: Similar definition to pymongo.collection.find method.
+        :param distinct_key: If not None, the key for which to get distinct results
+        :return: A QueryResults Iterable, which is somewhat like pymongo's
             cursor except that it performs mapping. In general, the dev does
             not need to concern himself with the form. It is sufficient to know
             that the results are in the form of an iterable of dicts.
         """
-
         if properties is not None:
             props, prop_dict = self._parse_properties(properties)
         else:
             props, prop_dict = None, None
 
         crit = self._parse_criteria(criteria) if criteria is not None else {}
+        #print("@@ mongo query = {} and fields = {}".format(crit, props))
         cur = self.collection.find(crit, fields=props,
                                    timeout=False).skip(index)
-        if limit is None:
-            return QueryResults(prop_dict, cur)
-        else:
+        if limit is not None:
             cur.limit(limit)
+        if distinct_key is not None:
+            cur = cur.distinct(distinct_key)
+            return QueryListResults(prop_dict, cur)
+        else:
             return QueryResults(prop_dict, cur)
 
     def _parse_properties(self, properties):
@@ -362,25 +372,10 @@ class QueryEngine(object):
                 prop_dict[p] = p.split(".")
         return props, prop_dict
 
-    def query_one(self, properties=None, criteria=None, index=0):
+    def query_one(self, *args, **kwargs):
+        """Return first document from :meth:`query`, with same parameters.
         """
-        Just like query, but return a single doc.
-
-        Args:
-            properties:
-                Properties to query for. Defaults to None which means all
-                supported properties.
-            criteria:
-                Criteria to query for as a dict.
-            index:
-                Similar definition to pymongo.collection.find method.
-
-        Returns:
-            A single dict containing the result.
-        """
-
-        for r in self.query(properties=properties, criteria=criteria,
-                            index=index, limit=1):
+        for r in self.query(*args, **kwargs):
             return r
         return None
 
@@ -500,6 +495,17 @@ class QueryResults(Iterable):
             yield self._mapped_result(r)
 
 
+class QueryListResults(QueryResults):
+    """Set of QueryResults on a list instead of a MongoDB cursor.
+    """
+
+    def clone(self):
+        return QueryResults(self._prop_dict, self._results[:])
+
+    def __len__(self):
+        return len(self._results)
+
+
 class QueryError(Exception):
     """
     Exception class for errors occuring during queries.
