@@ -4,7 +4,8 @@ from django.conf.urls import patterns, include, url
 from django.contrib import admin
 admin.autodiscover()
 
-urlpatterns = patterns('',
+urlpatterns = patterns(
+    '',
     # Examples:
     url(r'', include('matgendb.webui.home.urls')),
     url(r'rest', include('matgendb.webui.rest.urls')),
